%{
//File: arithmetic.y
//Author: Robert C. Broeckelmann Jr.
//Date:   7 Nov 2003
//Notes:
//	I stole this off the internet.  I've modified it heavily to suite my purposes, but
//	the original authors name and comments are still listed below.
//
// FILE: arithmetic.y
// Written by Michael Main (main@colorado.edu) -- Sep 11, 1998
// This is the complete bison file for evaluating arithmetic expressions.
// The input expression can contain double numbers, +(addition), 
// -(subtraction), *(multiplication), /(division), **(exponentiation),
// and parentheses to change the usual precedence. The expression
// ends with a single semi-colon.
// SEE ALSO: arithmetic.lex and makefile in ~main/proglang on nag.
//#include "stdafx.h"
#include <stdio.h>           // Provides sscanf
#include <stdlib.h>
#include <math.h>            // Provides pow
//#include "..\Unbounded_Stack.h"
#define YYSTYPE double				// Defines the type for semantic values.
char *yytext;			// The text of most recent token from the scanner.
int yylex( );			// Prototype for yylex,produced by the scanner.
int yyin;				// File Descriptor for file that is to be parsed...
int yy_scan_string(char *);
void yyerror(const char * s);		// Function to print an error message 
double answer;						// Value of the whole expression

//
//Data Structures needed to parse input...
//
//Unbounded_Stack<char> bracket_stack(100);
//Unbounded_Stack<int>  number_stack(100);

%}

%token    NUMBER PLUS MINUS TIMES DIVIDE EXP LEFT_BRACKET RIGHT_BRACKET CONSTANT INTEGER RATIONAL REAL COMPLEX IMAGINARY EQUAL ASSIGNMENT_EQUAL VARIABLE
%left     PLUS MINUS
%left     TIMES DIVIDE
%right    EXP

%start complete

%%
complete          : expr ';'
                  { answer = $1; }
                  ;
                            
expr              : expr PLUS expr
                  { $$ = $1 + $3; }
                  | expr MINUS expr
                  { $$ = $1 - $3; }
                  | expr TIMES expr
                  { $$ = $1 * $3; }
                  | expr DIVIDE expr
                  { $$ = $1 / $3; }
                  | expr EXP expr
                  { $$ = pow($1, $3); }
                  | LEFT_BRACKET expr RIGHT_BRACKET
                  { $$ = $2; }
                  | number
                  { 
					printf("I'm here\n");
					printf("answer=%lf\n", $$);
				  }
                  | variable
                  {
					 printf("I'm in expr variable.\n");
                  }
				;  
number			: NUMBER
				{ 
					printf("Inside number logic\n");
					sscanf(yytext, "%lf", &$$);
					printf("answer=%lf\n", $$);
				}
				;
variable		: VARIABLE
				{ 
					printf("Inside variable logic\n");
					$$=0.0; 
				}
				;
%%
/*****************************************************************************
* SUBROUTINES SECTION for arithmetic.y
* This version has only a simple main program that calls the parser and
* prints the calculated value of the expression.
*****************************************************************************/

double mainline(char *str)
{

	//Open tmp file containing string...
	
//	yy_scan_string(str);
	
	//
	//parse that file...
	//
    if (yyparse( ) != 0)
        fprintf(stdout,"Parsing failed.\n");
    else
        fprintf(stdout,"Parsing okay. Answer is: %f\n", answer );

    return answer;
}

void yyerror(const char* message)
{
    fprintf(stdout,"ERROR at token '%s': %s\n", yytext,message);
	//TRACE("ERROR at token '%s': %s", yytext,message);
}