%{
// FILE: arithmetic.y
// Written by Michael Main (main@colorado.edu) -- Sep 11, 1998
// This is the complete bison file for evaluating arithmetic expressions.
// The input expression can contain double numbers, +(addition), 
// -(subtraction), *(multiplication), /(division), **(exponentiation),
// and parentheses to change the usual precedence. The expression
// ends with a single semi-colon.
// SEE ALSO: arithmetic.lex and makefile in ~main/proglang on nag.
#include <iostream.h>        // C++ i/o library
#include <stdio.h>           // Provides sscanf
#include <math.h>            // Provides pow
#define YYSTYPE double       // Defines the type for semantic values.
extern char *yytext;         // The text of most recent token from the scanner.
int yylex( );                // Prototype for yylex,produced by the scanner.
void yyerror(const char * s);// Function to print an error message 
double answer;               // Value of the whole expression
%}

%token    NUMBER PLUS MINUS TIMES DIVIDE EXP
%left     PLUS MINUS
%left     TIMES DIVIDE
%right    EXP

%start complete

%%
complete          : expr ';'
                  { answer = $1; }
                  ;

expr              : expr PLUS expr
                  { $$ = $1 + $3; }
                  | expr MINUS expr
                  { $$ = $1 - $3; }
                  | expr TIMES expr
                  { $$ = $1 * $3; }
                  | expr DIVIDE expr
                  { $$ = $1 / $3; }
                  | expr EXP expr
                  { $$ = pow($1, $3); }
		  | '(' expr ')'
                  { $$ = $2; }
		  | NUMBER
                  { sscanf(yytext, "%lf", &$$); }
		  ;


%%
/*****************************************************************************
* SUBROUTINES SECTION for arithmetic.y
* This version has only a simple main program that calls the parser and
* prints the calculated value of the expression.
*****************************************************************************/

int main( )
{
    cout << "Starting parsing..." << endl;
    if (yyparse( ) != 0)
        cout << "Parsing failed." << endl;
    else
        cout << "Parsing okay. Answer is: " << answer << endl;
}

void yyerror(const char* message)
{
    cout << "ERROR at token '" << yytext << ":\n";
    cout << message << endl;
}
